import pytest

from asaaspy.exceptions import AsaasClientError
from asaaspy.schemas.v3.pix import (
    PixKeyViewSchema,
    StaticQRCodeSchema,
    StaticQRCodeViewSchema,
    PixTransactionSchema,
    QRCodePayViewSchema,
)


class TestPixResource:
    def test_list_keys(self, asaas_svc):
        return_data = asaas_svc.pix.list_keys()
        assert isinstance(return_data.data[0], PixKeyViewSchema)

    @pytest.mark.parametrize(
        "status, expected_amount_found", (("ACTIVE", 1), ("DELETED", 0))
    )
    def test_list_keys_with_filters(self, asaas_svc, status, expected_amount_found):
        return_data = asaas_svc.pix.list_keys(status=status)
        assert len(return_data.data) == expected_amount_found

    def test_get_key_by_id(self, asaas_svc):
        return_data = asaas_svc.pix.get_key_by_id(
            "564dbd99-8e86-4342-bafb-c778150a1869"
        )
        assert isinstance(return_data, PixKeyViewSchema)

    def test_create_random_key(self, asaas_svc):
        return_data = asaas_svc.pix.create_random_key()
        assert isinstance(return_data, PixKeyViewSchema)

    def test_create_static_qrcode(self, asaas_svc):
        request = StaticQRCodeSchema(
            addressKey="e3326671-e7d7-40bc-b54d-d8303f8140e8", format="ALL"
        )
        return_data = asaas_svc.pix.create_qrcode(request)
        assert isinstance(return_data, StaticQRCodeViewSchema)

    def test_delete_static_qrcode(self, asaas_svc):
        assert asaas_svc.pix.delete_qrcode("PRELUDIA00000000525658ASA")

    def test_delete_key_by_id_should_trigger_error_400(self, asaas_svc):
        with pytest.raises(AsaasClientError) as exc:
            asaas_svc.pix.delete_key("39006e1e-7c57-4944-ab22-10d957a0d1c9")

        assert exc.value.message == (
            "Sua conta possui autorização crítica habilitada. "
            "Para efetuar esta ação informe o código de confirmação."
        )

    # Payment
    def test_pay_qrcode(self, asaas_svc):
        request = PixTransactionSchema(
            **{
                "qrCode": {
                    "payload": "00020101021226820014br.gov.bcb.pix2560pix-h.asaas.com/qr/cobv/b8eb41f5-468b-4914-8297-662bef42b3f25204000053039865802BR5914Preludian Tech6009Guarulhos61080711500062070503***6304B90E"
                },
                "value": 10,
            }
        )

        response = asaas_svc.pix.pay_qrcode(request)
        assert isinstance(response, QRCodePayViewSchema)

    def test_get_transactions(self, asaas_svc):
        return_data = asaas_svc.pix.get_transactions()
        assert isinstance(return_data.data[0], QRCodePayViewSchema)

    def test_get_transactions_with_filter(self, asaas_svc):
        return_data = asaas_svc.pix.get_transactions(status="DONE")
        assert isinstance(return_data.data[0], QRCodePayViewSchema)

    def test_recover_transaction_by_id(self, asaas_svc):
        return_data = asaas_svc.pix.get_transaction(
            id="693f1933-a522-4c14-9f6e-7cd2c3030ca1"
        )
        assert isinstance(return_data, QRCodePayViewSchema)

    def test_cancel_scheduled_transaction_by_id(self, asaas_svc):
        return_data = asaas_svc.pix.cancel_scheduled_transaction(
            id="693f1933-a522-4c14-9f6e-7cd2c3030ca1"
        )
        assert isinstance(return_data, QRCodePayViewSchema)
        assert return_data.status == "CANCELLED"



from datetime import date
from enum import Enum
from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field

from asaaspy.schemas.base_fields import Date


def sanitize_to_json(value):
    if isinstance(value, Enum):
        return value.value

    if isinstance(value, date):
        return value.strftime("%Y-%m-%d")

    return value


class BaseSchema(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    def as_lean_dict(self):
        return self.model_dump(exclude_none=True, by_alias=True)


class PaginatedViewSchema(BaseSchema):
    object: Optional[str] = Field(default=None)
    hasMore: bool
    totalCount: int
    limit: int
    offset: int
    data: Any


class PaginatedQueryParams(BaseSchema):
    offset: Optional[int] = None
    limit: Optional[int] = None


class ViewItemSchema(BaseSchema):
    id: Optional[str] = None
    object: Optional[str] = None
    deleted: Optional[bool] = None
    dateCreated: Optional[Date] = None



from asaaspy.schemas.base import PaginatedViewSchema
from asaaspy.schemas.v3.bank import (
    AccountStatusViewSchema,
    BankAccountViewSchema,
    StatusEnum,
    TransactionItemViewSchema,
)


class TestBankResource:
    def test_get_bank_account_number(self, asaas_svc):
        account_info = asaas_svc.bank.get_account_number()
        assert isinstance(account_info, BankAccountViewSchema)
        assert isinstance(account_info.account, str)
        assert isinstance(account_info.accountDigit, str)
        assert isinstance(account_info.agency, str)

    def test_get_bank_balance(self, asaas_svc):
        balance_info = asaas_svc.bank.get_balance()

        assert isinstance(balance_info.balance, float)
        assert balance_info.balance == 701.79

    def test_get_bank_transactions(self, asaas_svc):
        return_data = asaas_svc.bank.get_transactions()
        assert isinstance(return_data, PaginatedViewSchema)
        assert isinstance(return_data.data[0], TransactionItemViewSchema)

    def test_get_bank_transactions_page_2(self, asaas_svc):
        return_data = asaas_svc.bank.get_transactions(offset=10, limit=10)
        assert isinstance(return_data, PaginatedViewSchema)
        assert isinstance(return_data.data[0], TransactionItemViewSchema)
        assert return_data.hasMore is True

    def test_get_bank_account_status(self, asaas_svc):
        return_data = asaas_svc.bank.get_account_status()
        assert isinstance(return_data, AccountStatusViewSchema)
        assert return_data.general == StatusEnum.PENDING.value



import factory

from asaaspy.schemas.v3.customer import CustomerSchema


class CustomerCreateSchemaFactory(factory.Factory):
    class Meta:
        model = CustomerSchema

    name = factory.Faker("name")
    cpfCnpj = factory.Faker("cpf", locale="pt_BR")



from asaaspy.client.base import AsaasHTTPClient
from asaaspy.client.resources.bank import BankResource
from asaaspy.client.resources.customer import CustomerResource
from asaaspy.client.resources.payment import PaymentResource
from asaaspy.client.resources.pix import PixResource
from asaaspy.client.resources.transfer import TransferResource


class AsaasService:
    def __init__(self, *args, **kwargs) -> None:
        """
        Inits a new AsaasPy Server instance.

        params:
            api_key(str): Asaas API Key to be used - required
            sandbox(bool): whether you use or not sandbox (default False)
        """
        client = AsaasHTTPClient(*args, **kwargs)

        self.customer: CustomerResource = CustomerResource(client)
        self.payment: PaymentResource = PaymentResource(client)
        self.bank: BankResource = BankResource(client)
        self.transfer: TransferResource = TransferResource(client)
        self.pix: PixResource = PixResource(client)



import logging
import os

import pytest
import vcr

from asaaspy.service import AsaasService


@pytest.fixture
def asaas_svc(caplog, mock_http_client):
    caplog.set_level(logging.INFO)

    return AsaasService(
        api_key=os.environ.get("ASAAS_SANDBOX_TOKEN"),
        sandbox=True,
    )


@pytest.fixture
def mock_http_client(request):
    root_path = request.config.rootdir
    with vcr.use_cassette(
        f"{root_path}/tests/asaas_sandbox_vcr.yml",
        serializer="json",
        record_mode="new_episodes",
        filter_headers=["access_token"],
        match_on=["method", "scheme", "host", "port", "path", "query", "body"],
    ):
        yield



import datetime

from pydantic import BeforeValidator, PlainSerializer
from typing_extensions import Annotated

Date = Annotated[
    datetime.date,
    BeforeValidator(
        lambda x: datetime.datetime.strptime(x, "%Y-%m-%d") if isinstance(x, str) else x
    ),
    PlainSerializer(lambda x: x.strftime("%Y-%m-%d")),
]

DateTime = Annotated[
    datetime.datetime,
    BeforeValidator(
        lambda x: datetime.datetime.strptime(x, "%Y-%m-%d %H:%M:%S")
        if isinstance(x, str)
        else x
    ),
    PlainSerializer(lambda x: x.strftime("%Y-%m-%d %H:%M:%S")),
]



from typing import Optional

from asaaspy.schemas.base import BaseSchema, PaginatedQueryParams, ViewItemSchema


class CustomerSchema(BaseSchema):
    name: str
    cpfCnpj: Optional[str] = None
    email: Optional[str] = None
    phone: Optional[str] = None
    mobilePhone: Optional[str] = None
    address: Optional[str] = None
    addressNumber: Optional[str] = None
    complement: Optional[str] = None
    province: Optional[str] = None
    postalCode: Optional[str] = None
    externalReference: Optional[str] = None
    notificationDisabled: Optional[bool] = None
    additionalEmails: Optional[str] = None
    municipalInscription: Optional[str] = None
    stateInscription: Optional[str] = None
    observations: Optional[str] = None
    groupName: Optional[str] = None
    company: Optional[str] = None

    city: Optional[str] = None
    cityName: Optional[str] = None
    state: Optional[str] = None
    country: Optional[str] = None


class CustomerViewSchema(CustomerSchema, ViewItemSchema):
    personType: Optional[str] = None
    canDelete: Optional[bool] = None
    canEdit: Optional[bool] = None
    cannotBeDeletedReason: Optional[str] = None
    cannotEditReason: Optional[str] = None


class CustomerSearchParams(PaginatedQueryParams):
    name: Optional[str] = None
    email: Optional[str] = None
    cpfCnpj: Optional[str] = None
    groupName: Optional[str] = None
    externalReference: Optional[str] = None



from enum import Enum
from typing import Literal, Optional

from asaaspy.schemas.base import BaseSchema


class BankAccountViewSchema(BaseSchema):
    agency: str
    account: str
    accountDigit: str


class BankBalanceViewSchema(BaseSchema):
    balance: float


class TransactionItemViewSchema(BaseSchema):
    object: str
    id: str
    value: float
    balance: float
    date: str
    description: str
    paymentId: str
    splitId: Optional[str] = None
    transferId: Optional[str] = None
    anticipationId: Optional[str] = None
    billId: Optional[str] = None
    invoiceId: Optional[str] = None
    paymentDunningId: Optional[str] = None
    creditBureauReportId: Optional[str] = None
    type: Literal[
        "PAYMENT_RECEIVED",
        "TRANSFER",
        "TRANSFER_FEE",
        "TRANSFER_REVERSAL",
        "REVERSAL",
        "PAYMENT_REVERSAL",
        "PAYMENT_REFUND_CANCELLED",
        "PAYMENT_FEE",
        "PAYMENT_FEE_REVERSAL",
        "PAYMENT_CUSTODY_BLOCK",
        "PAYMENT_CUSTODY_BLOCK_REVERSAL",
        "PHONE_CALL_NOTIFICATION_FEE",
        "PROMOTIONAL_CODE_CREDIT",
        "DEBIT",
        "DEBIT_REVERSAL",
        "DEBT_RECOVERY_NEGOTIATION_FINANCIAL_CHARGES",
        "RECEIVABLE_ANTICIPATION_GROSS_CREDIT",
        "RECEIVABLE_ANTICIPATION_FEE",
        "RECEIVABLE_ANTICIPATION_DEBIT",
        "RECEIVABLE_ANTICIPATION_PARTNER_SETTLEMENT",
        "BILL_PAYMENT",
        "BILL_PAYMENT_FEE",
        "BILL_PAYMENT_CANCELLED",
        "BILL_PAYMENT_FEE_CANCELLED",
        "BILL_PAYMENT_REFUNDED",
        "POSTAL_SERVICE_FEE",
        "INTERNAL_TRANSFER_DEBIT",
        "INTERNAL_TRANSFER_CREDIT",
        "INTERNAL_TRANSFER_REVERSAL",
        "CREDIT",
        "PARTIAL_PAYMENT",
        "PAYMENT_MESSAGING_NOTIFICATION_FEE",
        "PAYMENT_SMS_NOTIFICATION_FEE",
        "PAYMENT_DUNNING_REQUEST_FEE",
        "PAYMENT_DUNNING_RECEIVED_FEE",
        "PAYMENT_DUNNING_RECEIVED_IN_CASH_FEE",
        "PAYMENT_DUNNING_CANCELLATION_FEE",
        "CUSTOMER_COMMISSION_SETTLEMENT_CREDIT",
        "CUSTOMER_COMMISSION_SETTLEMENT_DEBIT",
        "PROMOTIONAL_CODE_DEBIT",
        "REFUND_REQUEST_FEE",
        "REFUND_REQUEST_CANCELLED",
        "REFUND_REQUEST_FEE_REVERSAL",
        "INVOICE_FEE",
        "PRODUCT_INVOICE_FEE",
        "CONSUMER_INVOICE_FEE",
        "ASAAS_CARD_RECHARGE",
        "ASAAS_CARD_RECHARGE_REVERSAL",
        "ASAAS_CARD_BALANCE_REFUND",
        "CHARGEBACK",
        "CHARGEBACK_REVERSAL",
        "ASAAS_CARD_BILL_PAYMENT",
        "ASAAS_CARD_BILL_PAYMENT_REFUND",
        "ASAAS_CARD_TRANSACTION",
        "ASAAS_CARD_CASHBACK",
        "ASAAS_CARD_CREDIT_VOUCHER",
        "ASAAS_CARD_CREDIT_VOUCHER_REFUND",
        "ASAAS_CARD_TRANSACTION_FEE",
        "ASAAS_CARD_TRANSACTION_REFUND",
        "ASAAS_CARD_TRANSACTION_FEE_REFUND",
        "ASAAS_CARD_TRANSACTION_PARTIAL_REFUND",
        "ASAAS_CARD_TRANSACTION_REFUND_CANCELLATION",
        "ASAAS_CARD_TRANSACTION_PARTIAL_REFUND_CANCELLATION",
        "ASAAS_MONEY_PAYMENT_COMPROMISED_BALANCE",
        "ASAAS_MONEY_PAYMENT_COMPROMISED_BALANCE_REFUND",
        "ASAAS_MONEY_PAYMENT_ANTICIPATION_FEE",
        "ASAAS_MONEY_PAYMENT_ANTICIPATION_FEE_REFUND",
        "ASAAS_MONEY_PAYMENT_FINANCING_FEE",
        "ASAAS_MONEY_PAYMENT_FINANCING_FEE_REFUND",
        "ASAAS_MONEY_TRANSACTION_CASHBACK",
        "ASAAS_MONEY_TRANSACTION_CASHBACK_REFUND",
        "ASAAS_MONEY_TRANSACTION_CHARGEBACK",
        "ASAAS_MONEY_TRANSACTION_CHARGEBACK_REVERSAL",
        "CHARGED_FEE_REFUND",
        "PIX_TRANSACTION_DEBIT",
        "PIX_TRANSACTION_DEBIT_REFUND",
        "PIX_TRANSACTION_CREDIT",
        "PIX_TRANSACTION_CREDIT_FEE",
        "PIX_TRANSACTION_CREDIT_REFUND",
        "PIX_TRANSACTION_CREDIT_REFUND_CANCELLATION",
        "PIX_TRANSACTION_DEBIT_FEE",
        "CREDIT_BUREAU_REPORT",
        "CONTRACTUAL_EFFECT_SETTLEMENT",
        "CONTRACTUAL_EFFECT_SETTLEMENT_REVERSAL",
        "FREE_PAYMENT_USE",
        "MOBILE_PHONE_RECHARGE",
        "REFUND_MOBILE_PHONE_RECHARGE",
        "CANCEL_MOBILE_PHONE_RECHARGE",
        "BACEN_JUDICIAL_LOCK",
        "BACEN_JUDICIAL_UNLOCK",
        "BACEN_JUDICIAL_TRANSFER",
        "INSTANT_TEXT_MESSAGE_FEE",
        "ASAAS_DEBIT_CARD_REQUEST_FEE",
        "ASAAS_PREPAID_CARD_REQUEST_FEE",
        "EXTERNAL_SETTLEMENT_CONTRACTUAL_EFFECT_BATCH_CREDIT",
        "EXTERNAL_SETTLEMENT_CONTRACTUAL_EFFECT_BATCH_REVERSAL",
        "CHILD_ACCOUNT_KNOWN_YOUR_CUSTOMER_BATCH_FEE",
        "CONTRACTED_CUSTOMER_PLAN_FEE",
        "ACCOUNT_INACTIVITY_FEE",
        "PAYMENT_ORIGIN_CHANNEL_FEE",
        "PAYMENT_SPLIT_DIVERGENCE_BLOCK",
        "PAYMENT_SPLIT_DIVERGENCE_BLOCK_REVERSAL",
        "RECEIVABLE_ANTICIPATION_PAYMENT_FEE",
        "ASAAS_MONEY_DONATION_DEBIT",
        "ASAAS_MONEY_DONATION_DEBIT_REFUND",
        "ASAAS_MONEY_PAYMENT_DEBIT",
        "ASAAS_MONEY_PAYMENT_DEBIT_REFUND",
        "CUSTOMER_COMMISSION_CHECKOUT",
        "RECEIVABLE_ANTICIPATION_CREDIT",
    ]


class StatusEnum(Enum):
    PENDING = "PENDING"
    APPROVED = "APPROVED"
    REJECTED = "REJECTED"
    AWAITING_APPROVAL = "AWAITING_APPROVAL"


class AccountStatusViewSchema(BaseSchema):
    id: str
    commercialInfo: StatusEnum
    bankAccountInfo: StatusEnum
    documentation: StatusEnum
    general: StatusEnum



from typing import Literal, Optional

from pydantic import Field

from asaaspy.schemas.base import BaseSchema, PaginatedQueryParams
from asaaspy.schemas.base_fields import Date, DateTime


class BankDetailViewSchema(BaseSchema):
    name: str
    ispb: str
    code: Optional[str] = None


class BankAccountViewSchema(BaseSchema):
    bank: BankDetailViewSchema
    ownerName: str
    cpfCnpj: str
    type: str
    accountName: Optional[str] = None
    agency: Optional[str] = None
    agencyDigit: Optional[str] = None
    account: Optional[str] = None
    accountDigit: Optional[str] = None
    pixAddressKey: Optional[str] = None


class TransferItemViewSchema(BaseSchema):
    object: str
    id: str
    value: float
    netValue: float
    transferFee: float
    dateCreated: Date
    status: Literal["PENDING", "BANK_PROCESSING", "DONE", "CANCELLED", "FAILED"]
    effectiveDate: Optional[DateTime] = Field(default=None)
    confirmedDate: Optional[Date] = Field(default=None)
    endToEndIdentifier: Optional[str] = Field(default=None)
    transactionReceiptUrl: Optional[str] = Field(default=None)
    operationType: Literal["PIX", "TED", "INTERNAL"]
    failReason: Optional[str]
    walletId: Optional[str]
    description: Optional[str]
    canBeCancelled: bool
    externalReference: Optional[str]
    authorized: bool
    scheduleDate: Optional[Date]
    type: str
    bankAccount: BankAccountViewSchema


class TransferFilterBy(PaginatedQueryParams):
    dateCreated_ge: Optional[Date] = Field(
        serialization_alias="dateCreated[ge]", default=None
    )
    dateCreated_le: Optional[Date] = Field(
        serialization_alias="dateCreated[le]", default=None
    )
    transferDate_ge: Optional[Date] = Field(
        serialization_alias="paymentDate[ge]", default=None
    )
    transferDate_le: Optional[Date] = Field(
        serialization_alias="paymentDate[le]", default=None
    )
    type: Optional[str] = None


class BankSchema(BaseSchema):
    code: str


class BankAccountTEDSchema(BaseSchema):
    bank: BankSchema
    ownerName: str
    accountName: Optional[str] = None
    ownerBirthDate: Optional[Date] = None
    cpfCnpj: Optional[str] = None
    agency: Optional[str] = None
    account: Optional[str] = None
    accountDigit: Optional[str] = None
    bankAccountType: Optional[Literal["CONTA_CORRENTE", "CONTA_POUPANCA"]] = None
    ispb: Optional[str] = None


class TransferSchema(BaseSchema):
    value: float
    operationType: Literal["PIX", "TED"] = "PIX"
    pixAddressKey: Optional[str] = None
    pixAddressKeyType: Literal["CPF", "CNPJ", "EMAIL", "PHONE", "EVP"]
    description: Optional[str] = None
    scheduleDate: Optional[Date] = None
    externalReference: Optional[str] = None
    bankAccount: Optional[BankAccountTEDSchema] = None



from json.decoder import JSONDecodeError

import httpx

from asaaspy.constants import APIBaseURL
from asaaspy.exceptions import AsaasClientError
from asaaspy.log import log
from asaaspy.schemas.base import PaginatedViewSchema


class AsaasHTTPClient:
    def __init__(self, api_key, sandbox=False) -> None:
        self._client_params = {
            "headers": {
                "Content-Type": "application/json",
                "User-Agent": "AsaasPy",
                "access_token": api_key,
            },
            "base_url": (
                APIBaseURL.SANDBOX if sandbox else APIBaseURL.PRODUCTION
            ).value,
        }

        log.info(
            f"Initialized AsaasClient in {'sandbox' if sandbox else 'production'} mode"
        )

    def get_http_client(self):
        log.info("Client instantiated")
        return httpx.Client(**self._client_params, timeout=30)


class AsaasResource:
    def __init__(self, asaas_client_instance):
        self._client = asaas_client_instance

    def get_client(self):
        return self._client.get_http_client()

    def call(self, *args, **kwargs):
        with self._client.get_http_client() as client:
            request = client.build_request(*args, **kwargs)
            response = client.send(request)

            if response.status_code not in [
                200,
                202,
                203,
                204,
                205,
                206,
                207,
                208,
            ]:  # TODO think in a better way
                try:
                    error_data = response.json()
                except JSONDecodeError:
                    error_data = None

                response_text = (
                    error_data["errors"][0]["description"]
                    if error_data
                    else response.text
                )
                raise AsaasClientError(
                    message=response_text,
                    status_code=response.status_code,
                    details=error_data,
                )

            return response.json()

    def get_list_response(
        self, response, data_response_class=dict
    ) -> PaginatedViewSchema:
        # TODO: Work on a better way to handle HTTP RESPONSES
        response = PaginatedViewSchema(**response)
        response.data = [data_response_class(**item) for item in response.data]

        return response



from asaaspy.client.base import AsaasResource
from asaaspy.schemas.v3.pix import (
    PixKeyFilterBy,
    PixKeyViewSchema,
    StaticQRCodeSchema,
    StaticQRCodeViewSchema,
    PixTransactionSchema,
    QRCodePayViewSchema,
)
from asaaspy.schemas.v3.pix import PixTransactionsFilterBySchema


class PixResource(AsaasResource):
    def list_keys(self, **filter_by):
        pix_key_filters = PixKeyFilterBy(**filter_by)
        return self.get_list_response(
            self.call(
                "GET", "v3/pix/addressKeys", params=pix_key_filters.as_lean_dict()
            ),
            PixKeyViewSchema,
        )

    def get_key_by_id(self, id: str) -> PixKeyViewSchema:
        response = self.call("GET", f"v3/pix/addressKeys/{id}")
        return PixKeyViewSchema(**response)

    def delete_key(self, id: str) -> PixKeyViewSchema:
        response = self.call("DELETE", f"v3/pix/addressKeys/{id}")
        return PixKeyViewSchema(**response)

    def create_random_key(self) -> PixKeyViewSchema:
        response = self.call("POST", "v3/pix/addressKeys", json={"type": "EVP"})
        return PixKeyViewSchema(**response)

    def create_qrcode(self, qrcode: StaticQRCodeSchema) -> StaticQRCodeViewSchema:
        response = self.call(
            "POST", "v3/pix/qrCodes/static", json=qrcode.as_lean_dict()
        )
        return StaticQRCodeViewSchema(**response)

    def delete_qrcode(self, id) -> bool:
        response = self.call("DELETE", f"v3/pix/qrCodes/static/{id}")
        return response.get("deleted", False)

    def pay_qrcode(self, qrcode: PixTransactionSchema) -> QRCodePayViewSchema:
        response = self.call("POST", "v3/pix/qrCodes/pay", json=qrcode.as_lean_dict())
        return QRCodePayViewSchema(**response)

    def decode_qrcode(self, id):
        raise NotImplementedError("Decode QR code not implemented")

    def get_transactions(self, **filter_by):
        filter_by = PixTransactionsFilterBySchema(**filter_by)
        return self.get_list_response(
            self.call("GET", "v3/pix/transactions", params=filter_by.as_lean_dict()),
            QRCodePayViewSchema,
        )

    def get_transaction(self, id) -> QRCodePayViewSchema:
        response = self.call("GET", f"v3/pix/transactions/{id}")
        return QRCodePayViewSchema(**response)

    def cancel_scheduled_transaction(self, id) -> QRCodePayViewSchema:
        response = self.call("POST", f"v3/pix/transactions/{id}/cancel")
        return QRCodePayViewSchema(**response)



from asaaspy.client.base import AsaasResource
from asaaspy.schemas.base import PaginatedViewSchema
from asaaspy.schemas.v3.payment import (
    PaymentCreateSchema,
    PaymentFilterBy,
    PaymentViewSchema,
)


class PaymentResource(AsaasResource):
    def create(self, payment: PaymentCreateSchema) -> PaymentViewSchema:
        response = self.call("POST", "/v3/payments", json=payment.as_lean_dict())
        return PaymentViewSchema(**response)

    def put(self, payment_id, payment: PaymentCreateSchema) -> PaymentViewSchema:
        response = self.call(
            "PUT", f"/v3/payments/{payment_id}", json=payment.as_lean_dict()
        )
        return PaymentViewSchema(**response)

    def get(self, payment_id) -> PaymentViewSchema:
        response = self.call("GET", f"/v3/payments/{payment_id}")
        return PaymentViewSchema(**response)

    def all(self, **payment_filter_by) -> PaginatedViewSchema:
        filter_by = PaymentFilterBy(**payment_filter_by) if payment_filter_by else None
        response = self.get_list_response(
            self.call(
                "GET",
                "v3/payments",
                params=filter_by.as_lean_dict() if filter_by else None,
            ),
            data_response_class=PaymentViewSchema,
        )
        return response

    def delete(self, payment_id) -> bool:
        response = self.call("DELETE", f"v3/payments/{payment_id}")
        return response.get("deleted", False)



from typing import Literal, Optional

from asaaspy.schemas.base import BaseSchema, PaginatedQueryParams
from asaaspy.schemas.base_fields import DateTime, Date


class QRCodeViewSchema(BaseSchema):
    encodedImage: str
    payload: str


class PixKeyViewSchema(BaseSchema):
    id: str
    key: str
    type: str
    status: Literal[
        "WAITING_ACTIVATION",
        "ACTIVE",
        "AWAITING_DELETION",
        "AWAITING_ACCOUNT_DELETION",
        "DELETED",
        "ERROR",
    ]
    dateCreated: DateTime
    canBeDeleted: bool
    cannotBeDeletedReason: Optional[str]
    qrCode: QRCodeViewSchema


class PixKeyFilterBy(PaginatedQueryParams):
    status: Optional[
        Literal[
            "AWAITING_ACTIVATION",
            "ACTIVE",
            "AWAITING_DELETION",
            "AWAITING_ACCOUNT_DELETION",
            "DELETED",
            "ERROR",
        ]
    ] = None
    statusList: Optional[str] = None


class StaticQRCodeSchema(BaseSchema):
    addressKey: str
    format: Literal["ALL", "IMAGE", "PAYLOAD"]
    description: Optional[str] = None
    value: Optional[float] = None
    expirationDate: Optional[DateTime] = None
    expirationSeconds: Optional[int] = None
    allowsMultiplePayments: Optional[bool] = None
    externalReference: Optional[str] = None


class StaticQRCodeViewSchema(BaseSchema):
    id: str
    encodedImage: str
    payload: str
    allowsMultiplePayments: Optional[bool] = None
    expirationDate: Optional[DateTime] = None
    externalReference: Optional[str] = None


# For Pix Payment
class QRCodePayloadSchema(BaseSchema):
    payload: str
    changeValue: Optional[float] = None


class PixTransactionSchema(BaseSchema):
    qrCode: QRCodePayloadSchema
    value: float
    description: Optional[str] = None
    scheduleDate: Optional[Date] = None


# For view
class PixOriginalTransactionViewSchema(BaseSchema):
    id: str
    value: float
    effectiveDate: Optional[Date] = None
    endToEndIdentifier: Optional[str] = None


class QRCodePayerViewSchema(BaseSchema):
    name: str
    cpfCnpj: str


class QRCodePayExternalAccountViewSchema(BaseSchema):
    ispb: int
    ispbName: Optional[str] = None
    name: Optional[str] = None
    cpfCnpj: Optional[str] = None
    addressKey: Optional[str] = None
    addressKeyType: Optional[Literal["CPF", "CNPJ", "EMAIL", "PHONE", "EVP"]] = None


class QRCodeInfoViewSchema(BaseSchema):
    payer: QRCodePayerViewSchema
    conciliationIdentifier: Optional[str] = None
    originalValue: Optional[float] = None
    dueDate: Optional[Date] = None
    interest: Optional[float] = None
    fine: Optional[float] = None
    discount: Optional[float] = None
    expirationDate: Optional[DateTime] = None


class QRCodePayViewSchema(BaseSchema):
    id: str
    value: float
    status: Literal[
        "AWAITING_BALANCE_VALIDATION",
        "AWAITING_INSTANT_PAYMENT_ACCOUNT_BALANCE",
        "AWAITING_CRITICAL_ACTION_AUTHORIZATION",
        "AWAITING_CHECKOUT_RISK_ANALYSIS_REQUEST",
        "AWAITING_CASH_IN_RISK_ANALYSIS_REQUEST",
        "SCHEDULED",
        "AWAITING_REQUEST",
        "REQUESTED",
        "DONE",
        "REFUSED",
        "CANCELLED",
    ]
    type: Literal[
        "DEBIT", "CREDIT", "CREDIT_REFUND", "DEBIT_REFUND", "DEBIT_REFUND_CANCELLATION"
    ]
    originType: Literal[
        "MANUAL",
        "ADDRESS_KEY",
        "STATIC_QRCODE",
        "DYNAMIC_QRCODE",
        "PAYMENT_INITIATION_SERVICE",
    ]
    endToEndIdentifier: Optional[str] = None
    qrCode: Optional[QRCodeInfoViewSchema] = None
    addressKeyType: Optional[Literal["CPF", "CNPJ", "EMAIL", "PHONE", "EVP"]] = None
    addressKey: Optional[str] = None
    originalTransaction: Optional[PixOriginalTransactionViewSchema] = None
    finality: Optional[Literal["WITHDRAWAL", "CHANGE"]] = None
    externalAccount: Optional[QRCodePayExternalAccountViewSchema] = None
    payment: Optional[str] = None
    canBeRefunded: Optional[bool] = None
    refundDisabledReason: Optional[str] = None
    chargedFeeValue: Optional[float] = None
    dateCreated: Optional[DateTime] = None
    transferId: Optional[str] = None
    externalReference: Optional[str] = None


class PixTransactionsFilterBySchema(PaginatedQueryParams):
    status: Optional[
        Literal[
            "AWAITING_BALANCE_VALIDATION",
            "AWAITING_INSTANT_PAYMENT_ACCOUNT_BALANCE",
            "AWAITING_CRITICAL_ACTION_AUTHORIZATION",
            "AWAITING_CHECKOUT_RISK_ANALYSIS_REQUEST",
            "AWAITING_CASH_IN_RISK_ANALYSIS_REQUEST",
            "SCHEDULED",
            "AWAITING_REQUEST",
            "REQUESTED",
            "DONE",
            "REFUSED",
            "CANCELLED",
        ]
    ] = None
    type: Optional[
        Literal[
            "DEBIT",
            "CREDIT",
            "CREDIT_REFUND",
            "DEBIT_REFUND",
            "DEBIT_REFUND_CANCELLATION",
        ]
    ] = None
    endToEndIdentifier: Optional[str] = None



from asaaspy.client.base import AsaasResource
from asaaspy.schemas.base import PaginatedQueryParams
from asaaspy.schemas.v3.bank import (
    AccountStatusViewSchema,
    BankAccountViewSchema,
    BankBalanceViewSchema,
    TransactionItemViewSchema,
)


class BankResource(AsaasResource):
    def get_account_number(self) -> BankAccountViewSchema:
        response = self.call("GET", "v3/myAccount/accountNumber")
        return BankAccountViewSchema(**response)

    def get_account_status(self):
        response = self.call("GET", "v3/myAccount/status")
        return AccountStatusViewSchema(**response)

    def get_balance(self):
        response = self.call("GET", "v3/finance/balance")
        return BankBalanceViewSchema(**response)

    def get_transactions(self, **pagination_kwargs):
        response = self.get_list_response(
            self.call(
                "GET",
                "v3/financialTransactions",
                params=PaginatedQueryParams(**pagination_kwargs).as_lean_dict(),
            ),
            data_response_class=TransactionItemViewSchema,
        )
        return response



from datetime import datetime
from enum import Enum
from typing import List, Literal, Optional

from pydantic import Field

from asaaspy.schemas.base import BaseSchema, PaginatedQueryParams, ViewItemSchema
from asaaspy.schemas.base_fields import Date


class BillingType(Enum):
    UNDEFINED = "UNDEFINED"
    BOLETO = "BOLETO"
    CREDIT_CARD = "CREDIT_CARD"
    PIX = "PIX"


class DiscountDetail(BaseSchema):
    value: int
    dueDateLimitDays: int
    type: Literal["FIXED", "PERCENTAGE"]


class InterestDetail(BaseSchema):
    value: float


class FineDetail(BaseSchema):
    value: int
    type: Literal["FIXED", "PERCENTAGE"]


class SplitDetail(BaseSchema):
    walletId: str
    fixedValue: Optional[float] = None
    percentualValue: Optional[float] = None
    totalFixedValue: Optional[float] = None


class CallbackDetail(BaseSchema):
    successUrl: str
    autoRedirect: bool = True


class PaymentCreateSchema(BaseSchema):
    customer: str
    billingType: BillingType
    value: float
    dueDate: Date
    description: Optional[str] = None
    daysAfterDueDateToRegistrationCancellation: Optional[int] = None
    externalReference: Optional[str] = None
    installmentCount: Optional[int] = None
    totalValue: Optional[float] = None
    installmentValue: Optional[float] = None
    discount: Optional[DiscountDetail] = None
    interest: Optional[InterestDetail] = None
    fine: Optional[FineDetail] = None
    postalService: Optional[bool] = None
    split: Optional[List[SplitDetail]] = None
    callback: Optional[CallbackDetail] = None


class RefundDetail(BaseSchema):
    dateCreated: Date
    status: Literal["PENDING", "CANCELLED", "DONE"]
    value: float
    description: Optional[str] = None
    transactionReceiptUrl: Optional[str] = None


class PaymentViewSchema(ViewItemSchema):
    customer: str
    dueDate: Date
    value: float
    netValue: float
    billingType: BillingType
    canBePaidAfterDueDate: bool
    confirmedDate: Optional[Date] = None
    pixQrCodeId: Optional[str] = None
    paymentLink: Optional[str] = None
    creditDate: Optional[Date] = None
    estimatedCreditDate: Optional[Date] = None
    lastInvoiceViewedDate: Optional[datetime] = None
    lastBankSlipViewedDate: Optional[datetime] = None
    pixTransaction: Optional[str] = None
    status: Literal[
        "PENDING",
        "RECEIVED",
        "CONFIRMED",
        "OVERDUE",
        "REFUNDED",
        "RECEIVED_IN_CASH",
        "REFUND_REQUESTED",
        "REFUND_IN_PROGRESS",
        "CHARGEBACK_REQUESTED",
        "CHARGEBACK_DISPUTE",
        "AWAITING_CHARGEBACK_REVERSAL",
        "DUNNING_REQUESTED",
        "DUNNING_RECEIVED",
        "AWAITING_RISK_ANALYSIS",
    ] = "PENDING"
    description: Optional[str] = None
    externalReference: Optional[str] = None
    originalValue: Optional[float] = None
    interestValue: Optional[float] = None
    originalDueDate: Optional[Date] = None
    paymentDate: Optional[Date] = None
    clientPaymentDate: Optional[Date] = None
    installmentNumber: Optional[int] = None
    transactionReceiptUrl: Optional[str] = None
    nossoNumero: Optional[str] = None
    invoiceUrl: Optional[str] = None
    bankSlipUrl: Optional[str] = None
    invoiceNumber: Optional[str] = None
    discount: Optional[DiscountDetail] = None
    fine: Optional[FineDetail] = None
    interest: Optional[InterestDetail] = None
    postalService: Optional[bool] = None
    anticipated: Optional[bool] = None
    anticipable: Optional[bool] = None
    refunds: Optional[List[RefundDetail]] = None
    custody: Optional[str] = (None,)
    canBePaidAfterDueDate: Optional[bool] = None


class PaymentFilterBy(PaginatedQueryParams):
    customer: Optional[str] = None
    customerGroupName: Optional[str] = None
    billingType: Optional[BillingType] = None
    status: Optional[
        Literal[
            "PENDING",
            "RECEIVED",
            "CONFIRMED",
            "OVERDUE",
            "REFUNDED",
            "RECEIVED_IN_CASH",
            "REFUND_REQUESTED",
            "REFUND_IN_PROGRESS",
            "CHARGEBACK_REQUESTED",
            "CHARGEBACK_DISPUTE",
            "AWAITING_CHARGEBACK_REVERSAL",
            "DUNNING_REQUESTED",
            "DUNNING_RECEIVED",
            "AWAITING_RISK_ANALYSIS",
        ]
    ] = None
    subscription: Optional[str] = None
    installment: Optional[str] = None
    externalReference: Optional[str] = None
    paymentDate: Optional[Date] = None
    invoiceStatus: Optional[str] = None
    estimatedCreditDate: Optional[Date] = None
    pixQrCodeId: Optional[str] = None
    anticipated: Optional[bool] = None
    dateCreated_ge: Optional[Date] = Field(
        serialization_alias="dateCreated[ge]", default=None
    )
    dateCreated_le: Optional[Date] = Field(
        serialization_alias="dateCreated[le]", default=None
    )
    paymentDate_ge: Optional[Date] = Field(
        serialization_alias="paymentDate[ge]", default=None
    )
    paymentDate_le: Optional[Date] = Field(
        serialization_alias="paymentDate[le]", default=None
    )
    estimatedCreditDate_ge: Optional[Date] = Field(
        serialization_alias="estimatedCreditDate[ge]", default=None
    )
    estimatedCreditDate_le: Optional[Date] = Field(
        serialization_alias="estimatedCreditDate[le]", default=None
    )
    dueDate_ge: Optional[Date] = Field(serialization_alias="dueDate[ge]", default=None)
    dueDate_le: Optional[Date] = Field(serialization_alias="dueDate[le]", default=None)
    user: Optional[str] = None  # email from user who create the charge



class AsaasClientError(Exception):
    def __init__(self, message: str, status_code: int, details: dict = None):
        self.message = message
        self.status_code = status_code
        self.details = details

    def __str__(self):
        return_message = f"HTTP {self.status_code}"

        if self.message:
            return_message += f": {self.message}"

        if self.details:
            return_message += f": {self.details}"

        return return_message



from datetime import date

import pytest

from asaaspy.exceptions import AsaasClientError
from asaaspy.schemas.v3.payment import (
    BillingType,
    PaymentCreateSchema,
    PaymentFilterBy,
    PaymentViewSchema,
)
from asaaspy.service import AsaasService


class TestAsaasCreatePayment:
    def test_create_minimal_payment(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        payment_create = PaymentCreateSchema(
            customer="cus_000006100224",
            billingType=BillingType.PIX,
            value=10.23,
            dueDate=date(year=2024, month=7, day=20),
        )

        payment_details = asaas_svc.payment.create(payment_create)

        assert payment_details.id.startswith("pay_")
        assert isinstance(payment_details, PaymentViewSchema)

    def test_create_payment_below_minimum_value(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        payment_create = PaymentCreateSchema(
            customer="cus_000006100224",
            billingType=BillingType.PIX,
            value=3,
            dueDate=date(year=2024, month=10, day=30),
        )

        with pytest.raises(AsaasClientError) as exc_info:
            asaas_svc.payment.create(payment_create)

        assert exc_info.value.message == (
            "O valor da cobrança (R$ 3,00) menos o valor do desconto (R$ 0,00) "
            "não pode ser menor que R$ 5,00."
        )
        assert exc_info.value.status_code == 400
        assert isinstance(exc_info.value.details, dict)


class TestAsaasGetPayment:
    def test_get_payment(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        payment = asaas_svc.payment.get("pay_egd03gp8vsz433au")

        assert payment.id == "pay_egd03gp8vsz433au"
        assert payment.value == 10.23
        assert payment.dueDate == date(year=2024, month=7, day=20)


class TestAsaasPutPayment:
    def test_put_payment(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        payment = asaas_svc.payment.put(
            "pay_atg6n271g1l6nzhm",
            PaymentCreateSchema(
                customer="cus_000006100224",
                billingType=BillingType.BOLETO,
                value=11.42,
                dueDate=date(year=2024, month=7, day=20),
            ),
        )

        assert payment.value == 11.42


class TestAsaasAllPayment:
    def test_all_payment(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        all_payments = asaas_svc.payment.all()

        assert all_payments.data is not None
        assert isinstance(all_payments.data[0], PaymentViewSchema)

    def test_all_payment_filter_dto(self):
        filter_by = PaymentFilterBy(
            dateCreated_ge=date(2024, 7, 19), dueDate_le=date(2024, 8, 3)
        ).as_lean_dict()

        assert {
            "dateCreated[ge]": "2024-07-19",
            "dueDate[le]": "2024-08-03",
        } == filter_by

    def test_all_payment_with_filter(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        filtered_payments = asaas_svc.payment.all(status="RECEIVED")

        assert filtered_payments.totalCount == 2


class TestAsaasDeletePayment:
    def test_delete_payment(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        has_deleted = asaas_svc.payment.delete(payment_id="pay_vtkxai01k8o8jo55")

        assert has_deleted


class TestAsaasGetWrongPayment:
    def test_get_payment(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        with pytest.raises(AsaasClientError) as exc_info:
            asaas_svc.payment.get("pay_that_not_exists")

        assert exc_info.value.status_code == 404
        assert exc_info.value.message == ""
        assert exc_info.value.details is None



from asaaspy.client.base import AsaasResource
from asaaspy.schemas.v3.transfer import (
    TransferFilterBy,
    TransferItemViewSchema,
    TransferSchema,
)


class TransferResource(AsaasResource):
    def get_transfers(self, **transfer_filter_by):
        transfer_filter_by = TransferFilterBy(**transfer_filter_by)

        response = self.get_list_response(
            self.call(
                "GET",
                "v3/transfers",
                params=transfer_filter_by.as_lean_dict()
                if transfer_filter_by
                else None,
            ),
            data_response_class=TransferItemViewSchema,
        )
        return response

    def get_by_id(self, transfer_id) -> TransferItemViewSchema:
        response = self.call("GET", f"v3/transfers/{transfer_id}")
        return TransferItemViewSchema(**response)

    def create(self, transfer: TransferSchema) -> TransferItemViewSchema:
        response = self.call("POST", "v3/transfers", json=transfer.as_lean_dict())
        return TransferItemViewSchema(**response)

    def delete(self, id: str) -> TransferItemViewSchema:
        response = self.call("DELETE", f"v3/transfers/{id}/cancel")
        return TransferItemViewSchema(**response)



import pytest

from asaaspy.schemas.v3.customer import CustomerSchema, CustomerViewSchema
from asaaspy.service import AsaasService


class TestAsaasGetCustomer:
    def test_get_all_customers(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc
        all_customers = asaas_svc.customer.all()

        assert all_customers.totalCount == 3
        assert isinstance(all_customers.data[0], CustomerViewSchema)

    def test_get_customer(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        CUSTOMER_ID = "cus_000006102042"
        customer = asaas_svc.customer.get(customer_id=CUSTOMER_ID)

        assert customer.id == CUSTOMER_ID

    @pytest.mark.parametrize(
        "extra_params",
        ({"name": "Pedro"}, {"name": "Lucas", "email": "joaodascouves1@gmail.com"}),
    )
    def test_get_all_customers_with_filter(self, asaas_svc, extra_params):
        asaas_svc: AsaasService = asaas_svc

        all_customers = asaas_svc.customer.all()

        assert all_customers.hasMore is False


class TestAsaasCreateCustomer:
    def test_create_customer(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        customer = CustomerSchema(name="Karen Baldwin", cpfCnpj="458.907.623-38")
        new_customer = asaas_svc.customer.create(customer)

        assert new_customer.id is not None

    def test_create_customer_minimal(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        customer = CustomerSchema(name="Empresa Apenas com o nome")
        new_customer = asaas_svc.customer.create(customer)

        assert new_customer.id.startswith("cus_")


class TestAsaasUpdateCustomer:
    def test_update_customer(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        customer = CustomerSchema(name="Karen Rizzo", cpfCnpj="458.907.623-38")
        new_customer = asaas_svc.customer.update(
            customer_id="cus_000006100224", customer=customer
        )

        assert new_customer.id is not None
        assert new_customer.name == "Karen Rizzo"


class TestAsaasDeleteCustomer:
    def test_delete(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        customer = CustomerSchema(
            name="Usuario a Ser Excluido", cpfCnpj="083.138.660-63"
        )
        new_customer = asaas_svc.customer.create(customer)

        has_deleted = asaas_svc.customer.delete(id=new_customer.id)

        assert has_deleted



from asaaspy.schemas.v3.transfer import TransferItemViewSchema, TransferSchema


class TestPaymentResource:
    def test_get_all_payments(self, asaas_svc):
        return_data = asaas_svc.transfer.get_transfers()
        assert isinstance(return_data.data[0], TransferItemViewSchema)

    def test_get_transfer_by_id(self, asaas_svc):
        return_data = asaas_svc.transfer.get_by_id(
            "93cbacf4-3bc3-4fd8-802a-f7fdc41922ea"
        )
        assert isinstance(return_data, TransferItemViewSchema)

    def test_create_payment(self, asaas_svc):
        transfer_request = TransferSchema(
            value=100,
            pixAddressKey="cliente-a00005@pix.bcb.gov.br",
            pixAddressKeyType="EMAIL",
        )
        return_data = asaas_svc.transfer.create(transfer_request)
        assert isinstance(return_data, TransferItemViewSchema)

    def test_delete_payment(self, asaas_svc):
        return_data = asaas_svc.transfer.delete(
            id="67049713-a41a-458a-aed4-729babb675ae"
        )
        assert isinstance(return_data, TransferItemViewSchema)
        assert return_data.status == "CANCELLED"



from asaaspy.client.base import AsaasResource
from asaaspy.schemas.v3.customer import (
    CustomerSchema,
    CustomerSearchParams,
    CustomerViewSchema,
)


class CustomerResource(AsaasResource):
    def all(
        self,
        name=None,
        email=None,
        cpf_cnpj=None,
        group_name=None,
        external_reference=None,
        **kwargs,
    ):
        response = self.get_list_response(
            self.call(
                "GET",
                "v3/customers",
                params=CustomerSearchParams(
                    **{
                        "name": name,
                        "email": email,
                        "cpfCnpj": cpf_cnpj,
                        "groupName": group_name,
                        "externalReference": external_reference,
                        **kwargs,
                    }
                ).as_lean_dict(),
            ),
            data_response_class=CustomerViewSchema,
        )
        return response

    def get(self, customer_id):
        response = self.call("GET", f"v3/customers/{customer_id}")
        return CustomerViewSchema(**response)

    def create(self, customer: CustomerSchema) -> CustomerViewSchema:
        response = self.call("POST", "v3/customers", json=customer.as_lean_dict())
        return CustomerViewSchema(**response)

    def update(self, customer_id, customer: CustomerSchema) -> CustomerViewSchema:
        response = self.call(
            "PUT", f"v3/customers/{customer_id}", json=customer.as_lean_dict()
        )
        return CustomerViewSchema(**response)

    def delete(self, id: str) -> bool:
        response = self.call("DELETE", f"v3/customers/{id}")
        return response.get("deleted", False)
